#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>
#include <syscall.h>

#include <sys/stat.h>
#include <sys/types.h>

#include <time.h>
#include <errno.h>

static char my_name[1024];

#define FOPE_AGE	(1 << 0)
#define FOPE_SPENT	(1 << 1)

#define MODIFY	(1 << 0)
#define ACCESS	(1 << 1)
#define CHANGE	(1 << 2)

#define ORDER_SEC	(1 << 0)
#define ORDER_MIN	(1 << 1)
#define ORDER_HOUR	(1 << 2)
#define ORDER_DAY	(1 << 3)

#define ORDER_DEFAULT	ORDER_MIN

//#define DEBUG
#if defined(DEBUG)
  #define debug(fmt, args...)	printf(fmt, ##args)
#else
  #define debug(fmt, args...)
#endif

#define STR_VERSION	"1.00"

void Usage(void)
{
	printf("\tage Ver.%s\n", STR_VERSION);
	printf("\n\t=== Usage ===\n");
	printf("\tfile age check program\n");
	printf("\tmage   : get spent time from last file modify.\n");
	printf("\taage   : get spent time from last file access.\n");
	printf("\tcage   : get spent time from last file change.\n");
	printf("\tmspent : judge specified time was spent from last file modify or not.\n");
	printf("\taspent : judge specified time was spent from last file access or not.\n");
	printf("\tcspant : judge specified time was spent from last file change or not.\n");
	printf("\n");
	printf("\tmspent, aspent, cspent commands return 0, if specified time was spent.\n");

	printf("\n");
	printf("\t=== Options === \n");
	printf("\t -f : specify file name\n");
	printf("\t -t : specify spent time to judge\n");
	printf("\t -o : specify time's order(sec, min, hour, day)\n");
	printf("\n");
	printf("\t=== Use like below ===\n");
	printf("\tget what hour was spent from last modify\n");
	printf("\t  age mtime -f test -o hour\n");
	printf("\n");
	printf("\tjudge 1 hour was spent or not from last modify\n");
	printf("\t  age mspent -f test -t 1 -o hour\n");
}

unsigned long long get_age(const char *filename, int flag)
{
	debug("%s : entered\n", __FUNCTION__);

	struct stat buf_stat;
	time_t present_time;

	if(stat(filename, &buf_stat) != 0)
	{
		printf("Error : %s\n", strerror(errno));
		return -1;
	}

	time(&present_time);
	if(present_time < buf_stat.st_mtime)
	{
		return -1;
	}

	switch(flag)
	{
	case MODIFY:
		return (unsigned long long)(present_time - buf_stat.st_mtime);
		break;
	case ACCESS:
		return (unsigned long long)(present_time - buf_stat.st_atime);
		break;
	case CHANGE:
		return (unsigned long long)(present_time - buf_stat.st_ctime);
		break;
	}
	return 0;
}

int main(int argc, char *argv[])
{
	char *pmy_name = NULL;
	int OPE_FLAG = 0;
	unsigned long long file_age = 0;
	char filename[1024];
	int opt;
	unsigned long long target_time = 0;
	unsigned long long target_age = 0;
	int order = ORDER_DEFAULT;

	if(argc <= 1)
	{
		Usage();
		return -1;
	}

	memset(my_name, 0, sizeof(my_name));
	pmy_name = basename(argv[0]);
	if((strlen(pmy_name) == (strlen("age"))) && (strncmp(pmy_name, "age", strlen("age")) == 0))
	{
		pmy_name = argv[1];
	}
	strcpy(my_name, pmy_name);

	memset(filename, 0, sizeof(filename));

	while((opt = getopt(argc, argv, "f:t:o:h")) != -1)
	{
		switch(opt)
		{
		case 'f':
			strcpy(filename, optarg);
			break;
		case 't':
			target_time = (unsigned long long)atoll(optarg);
			break;
		case 'o':
			if((strlen(optarg) == strlen("sec")) && (strncmp(optarg, "sec", strlen("sec")) == 0))
			{
				order = ORDER_SEC;
				debug("order = ORDER_SEC\n");
			}
			else if((strlen(optarg) == strlen("min")) && (strncmp(optarg, "min", strlen("min")) == 0))
			{
				order = ORDER_MIN;
				debug("order = ORDER_MIN\n");
			}
			else if((strlen(optarg) == strlen("hour")) && (strncmp(optarg, "hour", strlen("hour")) == 0))
			{
				order = ORDER_HOUR;
				debug("order = ORDER_HOUR\n");
			}
			else if((strlen(optarg) == strlen("day")) && (strncmp(optarg, "day", strlen("day")) == 0))
			{
				order = ORDER_DAY;
				debug("order = ORDER_DAY\n");
			}
			else
			{
				Usage();
				return -1;
			}
			break;
		case 'h':
			Usage();
			return -1;
			break;
		default:
			break;
		}
	}

	if(strlen(filename) < 1)
	{
		printf("You must specify file name like -f test\n");
		Usage();
		return -1;
	}

	if((strlen(my_name) == strlen("mage")) && (strncmp(my_name, "mage", strlen("mage")) == 0))
	{
		if((file_age = get_age(filename, MODIFY)) == 0)
		{
			return -1;
		}
		OPE_FLAG = FOPE_AGE;
	}
	else if((strlen(my_name) == strlen("aage")) && (strncmp(my_name, "aage", strlen("aage")) == 0))
	{
		if((file_age = get_age(filename, ACCESS)) == 0)
		{
			return -1;
		}
		OPE_FLAG = FOPE_AGE;
	}
	else if((strlen(my_name) == strlen("cage")) && (strncmp(my_name, "cage", strlen("cage")) == 0))
	{
		if((file_age = get_age(filename, CHANGE)) == 0)
		{
			return -1;
		}
		OPE_FLAG = FOPE_AGE;
	}
	else if((strlen(my_name) == strlen("mspent")) && (strncmp(my_name, "mspent", strlen("mspent")) == 0))
	{
		if(target_time == 0)
		{
			printf("You must specify spent time like -t 3600\n");
			Usage();
			return -1;
		}

		if((file_age = get_age(filename, MODIFY)) == 0)
		{
			return -1;
		}
		OPE_FLAG = FOPE_SPENT;
	}
	else if((strlen(my_name) == strlen("aspent")) && (strncmp(my_name, "aspent", strlen("aspent")) == 0))
	{
		if(target_time == 0)
		{
			printf("You must specify spent time like -t 3600\n");
			Usage();
			return -1;
		}

		if((file_age = get_age(filename, ACCESS)) == 0)
		{
			return -1;
		}
		OPE_FLAG = FOPE_SPENT;
	}
	else if((strlen(my_name) == strlen("cspent")) && (strncmp(my_name, "cspent", strlen("cspent")) == 0))
	{
		if(target_time == 0)
		{
			printf("You must specify spent time like -t 3600\n");
			Usage();
			return -1;
		}

		if((file_age = get_age(filename, CHANGE)) == 0)
		{
			return -1;
		}
		OPE_FLAG = FOPE_SPENT;
	}

	debug("file_age = %llu\n", file_age);

	if(file_age == (unsigned long long) -1)
	{
		fprintf(stderr, "file %s is manipulated in the future! or not exist.\n", filename);
		return 0;
	}

	switch(OPE_FLAG)
	{
	case FOPE_AGE:

		switch(order)
		{
		case ORDER_SEC:
			printf("%llu\n", file_age);
			break;
		case ORDER_MIN:
			printf("%llu\n", (file_age / 60));
			break;
		case ORDER_HOUR:
			printf("%llu\n", (file_age / 60 / 60));
			break;
		case ORDER_DAY:
			printf("%llu\n", (file_age / 60 / 60 / 24));
			break;
		}
		return 0;
		break;
	case FOPE_SPENT:
		switch(order)
		{
		case ORDER_SEC:
			target_age = target_time;
			break;
		case ORDER_MIN:
			target_age = target_time * 60;
			break;
		case ORDER_HOUR:
			target_age = target_time * 60 * 60;
			break;
		case ORDER_DAY:
			target_age = target_time * 60 * 60 * 24;
			break;
		}

		if(file_age >= target_age)
		{
			return 0;
		}
		else
		{
			return -1;
		}
		return -1;
		break;
	default:
		Usage();
		return -1;
		break;
	}

	return -1;
}
